--  with Ada.Text_IO;
--  with Ada.Integer_Text_IO;
--  with Ada.Float_Text_IO;
--
--  procedure Modbus00 is
--     -- Display the average of two numbers entered by the user
--     A : Integer;
--     B : Integer;
--     M : Float;
--  begin
--     Ada.Text_IO.Put_Line (Item => "Enter two integers.");
--     Ada.Integer_Text_IO.Get (Item => A);
--     Ada.Integer_Text_IO.Get (Item => B);
--
--     if A > B then
--        Ada.Text_IO.Put_Line (Item => "Hey ! Did you know that A > B ?");
--     elsif A < B then
--        Ada.Text_IO.Put_Line (Item => "Hey ! Did you know that A < B ?");
--     else
--        Ada.Text_IO.Put_Line (Item => "Hey ! Did you know that A = B ?");
--     end if;
--
--     M := Float(A + B) / 2.0;
--
--     Ada.Text_IO.New_Line;
--     Ada.Text_IO.Put (Item => "The average of your integers is ");
--     Ada.Float_Text_IO.Put (Item => M,
--                            Fore =>1,
--                            Aft => 2,
--                            Exp => 0);
--     Ada.Text_IO.New_Line;
--  end Modbus00;

with Ada.Integer_Text_IO;
with Ada.Unchecked_Conversion;
with Ada.Streams;
with Ada.IO_Exceptions;
with GNAT.Sockets;
use type Ada.Streams.Stream_Element_Offset;

package body Modbus00 is

   subtype Function_Code_Type_Buffer is Ada.Streams.Stream_Element;
   function Function_Code_Type_To_Function_Code_Type_Buffer is
     new Ada.Unchecked_Conversion (Function_Code_Type, Function_Code_Type_Buffer);
   function Function_Code_Type_Buffer_To_Function_Code_Type is
     new Ada.Unchecked_Conversion (Function_Code_Type_Buffer, Function_Code_Type);

   subtype Byte_Buffer is Ada.Streams.Stream_Element_Array (1 .. 1);
   function Byte_To_Byte_Buffer is new Ada.Unchecked_Conversion (Byte, Byte_Buffer);
   function Byte_Buffer_To_Byte is new Ada.Unchecked_Conversion (Byte_Buffer, Byte);

   subtype Word_Buffer is Ada.Streams.Stream_Element_Array (1 .. 2);
   function Word_To_Word_Buffer is new Ada.Unchecked_Conversion (Word, Word_Buffer);
   function Word_Buffer_To_Word is new Ada.Unchecked_Conversion (Word_Buffer, Word);

   -- MBAP_Header
   type MBAP_Header_Type is
      record
         -- Identification of a MODBUS Request / Response transaction.
         Transaction_Identifier : Word;
         -- 0 = MODBUS protocol
         Protocol_Identifier :Word;
         -- Number of following bytes
         Length : Word;
         -- Identification of a remote slave connected
         -- on a serial line or on other buses.
         Unit_Identifier : Byte;
      end record;

   MBAP_Header : MBAP_Header_Type := (
                                      Transaction_Identifier => 0,
                                      Protocol_Identifier    => 0,
                                      Length                 => 0,
                                      Unit_Identifier        => 255
                                     );

   function MBAP_Header_Length (Item : In Protocol_Data_Unit) return Word is
      Length : Word;
   begin
      Length := 1;
      case Item.Function_Code is
         when Read_Coils =>
            Length := Length + 2 + 2;
         when Read_Discrete_Inputs =>
            Length := Length + 2 + 2;
         when Read_Holding_Registers =>
            Length := Length + 2 + 2;
         when Write_Multiple_Registers =>
            Length := Length + 2 + 2;
      end case;

      return Length;
   end MBAP_Header_Length;

   function MBTCP_ADU_Length (MBAP_Header_Length : In Word) return Word is
      Length : Word;
   begin
      Length := 2 + 2 + 2 + 1 + MBAP_Header_Length;

      return Length;
   end MBTCP_ADU_Length;

   procedure Send (
                   Stream : not null access Ada.Streams.Root_Stream_Type'Class;
                   Item : In Protocol_Data_Unit) is
      MBAPH_Length : Word := MBAP_Header_Length(Item => Item);
      MBTCPADU_Length : Word := MBTCP_ADU_Length(MBAPH_Length);
      Buffer : Ada.Streams.Stream_Element_Array
        (1 .. Ada.Streams.Stream_Element_Offset (MBTCPADU_Length));
      WordBuffer1 : Word_Buffer;
      WordBuffer2 : Word_Buffer;
   begin
      MBAP_Header.Transaction_Identifier := MBAP_Header.Transaction_Identifier + 1;
      MBAP_Header.Length := MBAPH_Length;

      WordBuffer1 := Word_To_Word_Buffer(MBAP_Header.Transaction_Identifier);
      Buffer(1) := WordBuffer1(2);
      Buffer(2) := WordBuffer1(1);

      WordBuffer1 := Word_To_Word_Buffer(MBAP_Header.Protocol_Identifier);
      Buffer(3) := WordBuffer1(2);
      Buffer(4) := WordBuffer1(1);

      WordBuffer1 := Word_To_Word_Buffer(MBAP_Header.Length);
      Buffer(5) := WordBuffer1(2);
      Buffer(6) := WordBuffer1(1);

      Buffer(7) := Ada.Streams.Stream_Element(MBAP_Header.Unit_Identifier);

      Buffer(8) := Function_Code_Type_To_Function_Code_Type_Buffer(Item.Function_Code);
--                 Read_Coils_Starting_Address : Word;
--                 Quantity_Of_Coils : Read_Coils_Quantity;
--              when Read_Discrete_Inputs =>
--                 Read_Discrete_Inputs_Starting_Address : Word;
--                 Quantity_Of_Inputs : Read_Discrete_Inputs_Quantity;
--              when Read_Holding_Registers =>
--                 Read_Holding_Registers_Starting_Address : Word;
--                 Quantity_Of_Registers_To_Read : Read_Holding_Registers_Quantity;
--              when Write_Multiple_Registers =>
--                 Write_Multiple_Registers_Starting_Address : Word;
--                 Quantity_Of_Registers_To_Write : Write_Multiple_Registers_Quantity;

      case Item.Function_Code is
         when Read_Coils =>
            WordBuffer1 := Word_To_Word_Buffer(Item.Read_Coils_Starting_Address);
         when Read_Discrete_Inputs =>
            WordBuffer1 := Word_To_Word_Buffer(Item.Read_Discrete_Inputs_Starting_Address);
         when Read_Holding_Registers =>
            WordBuffer1 := Word_To_Word_Buffer(Item.Read_Holding_Registers_Starting_Address);
            WordBuffer2 := Word_To_Word_Buffer(Word(Item.Quantity_Of_Registers_To_Read));
         when Write_Multiple_Registers =>
            WordBuffer1 := Word_To_Word_Buffer(Item.Write_Multiple_Registers_Starting_Address);
      end case;
      Buffer(9)  := WordBuffer1(2);
      Buffer(10) := WordBuffer1(1);

      Buffer(11) := WordBuffer2(2);
      Buffer(12) := WordBuffer2(1);
      Stream.Write(Buffer);
   end Send;

   procedure Receive (
                   Stream : not null access Ada.Streams.Root_Stream_Type'Class;
                   Item : out Protocol_Data_Unit) is
      WordBuffer1 : Word_Buffer;
      WordBuffer2 : Word_Buffer;
      Last        : Ada.Streams.Stream_Element_Offset;
      ByteBuffer  : Byte_Buffer;
      aWord       : Word;
   begin
      Stream.Read(WordBuffer1, Last);
      WordBuffer2(1) := WordBuffer1(2);
      WordBuffer2(2) := WordBuffer1(1);
      aWord := Word_Buffer_To_Word(WordBuffer2);
      if (aWord /= MBAP_Header.Transaction_Identifier) then
         raise Ada.IO_Exceptions.Data_Error;
      end if;

      Stream.Read(WordBuffer1, Last);
      WordBuffer2(1) := WordBuffer1(2);
      WordBuffer2(2) := WordBuffer1(1);
      aWord := Word_Buffer_To_Word(WordBuffer2);
      if (aWord /= MBAP_Header.Protocol_Identifier) then
         raise Ada.IO_Exceptions.Data_Error;
      end if;

      Stream.Read(WordBuffer1, Last);
      WordBuffer2(1) := WordBuffer1(2);
      WordBuffer2(2) := WordBuffer1(1);
      aWord := Word_Buffer_To_Word(WordBuffer2);
      -- length should at least be 3 bytes :
      -- MBAP_Header.Unit_Identifier
      -- Item.Function_Code or Exception
      -- Exception
      if (aWord < 3) then
         raise Ada.IO_Exceptions.Data_Error;
      end if;

      Stream.Read(ByteBuffer, Last);
      if (Byte_Buffer_To_Byte(ByteBuffer) /= MBAP_Header.Unit_Identifier) then
         raise Ada.IO_Exceptions.Data_Error;
      end if;

      Stream.Read(ByteBuffer, Last);
      if (Function_Code_Type_Buffer_To_Function_Code_Type(ByteBuffer(1)) /= Item.Function_Code) then
         raise Ada.IO_Exceptions.Data_Error;
      end if;

      Stream.Read(ByteBuffer, Last); -- Byte count

      declare
         DataBuffer  : Ada.Streams.Stream_Element_Array (1 .. Ada.Streams.Stream_Element_Offset(Byte_Buffer_To_Byte(ByteBuffer)));
      begin
         Stream.Read(DataBuffer, Last);
         if (Last /= Ada.Streams.Stream_Element_Offset(Byte_Buffer_To_Byte(ByteBuffer))) then
            raise Ada.IO_Exceptions.Data_Error;
         end if;
      end;
   end Receive;

   procedure Do_It is
      Client  : GNAT.Sockets.Socket_Type;
      Address : GNAT.Sockets.Sock_Addr_Type;
      Channel : GNAT.Sockets.Stream_Access;

      Read_Holding_Registers_PDU : Protocol_Data_Unit (Read_Holding_Registers);

   begin
--        Ada.Integer_Text_IO.Put(Item => Byte'Size);
--        Ada.Integer_Text_IO.Put(Item => Word'Size);
--        Ada.Integer_Text_IO.Put(Item => Standard'Storage_Unit);
--        Ada.Integer_Text_IO.Put(Item => Standard'Address_Size);

      GNAT.Sockets.Initialize;  -- initialize a specific package
      GNAT.Sockets.Create_Socket (Client);
      Address.Addr := GNAT.Sockets.Inet_Addr("127.0.0.1");
      Address.Port := 1502;
      GNAT.Sockets.Connect_Socket (Client, Address);
      Channel := GNAT.Sockets.Stream (Client);

      Read_Holding_Registers_PDU.Read_Holding_Registers_Starting_Address := 0;
      Read_Holding_Registers_PDU.Quantity_Of_Registers_To_Read := 1;

      for I in 1 .. 10 loop
         Protocol_Data_Unit'Write (Channel, Read_Holding_Registers_PDU);
         Protocol_Data_Unit'Read (Channel, Read_Holding_Registers_PDU);
      end loop;

   end Do_It;

end Modbus00;



