with Ada.Streams;
package Modbus00 is

   type Byte is mod 2 **  8; -- Byte'Size is 8
   type Word is mod 2 ** 16; -- Word'Size is 16

   type Function_Code_Type is
     (
      Read_Coils,
      Read_Discrete_Inputs,
      Read_Holding_Registers,
      Write_Multiple_Registers
     );
   for Function_Code_Type'Size use 8;
   for Function_Code_Type use
     (
      Read_Coils               => 16#01#,
      Read_Discrete_Inputs     => 16#02#,
      Read_Holding_Registers   => 16#03#,
      Write_Multiple_Registers => 16#10#
     );

   type Read_Coils_Quantity is range 1 .. 2000;
   for Read_Coils_Quantity'Size use Word'Size;

   type Read_Discrete_Inputs_Quantity is range 1 .. 2000;
   for Read_Discrete_Inputs_Quantity'Size use Word'Size;

   type Read_Holding_Registers_Quantity is range 1 .. 125;
   for Read_Holding_Registers_Quantity'Size use Word'Size;

   type Write_Multiple_Registers_Quantity is range 1 .. 123;
   for Write_Multiple_Registers_Quantity'Size use Word'Size;

   type PDU_Data is array (0 .. 252) of Byte;

   type Protocol_Data_Unit (Function_Code : Function_Code_Type) is
      record
         case Function_Code is
            when Read_Coils =>
               Read_Coils_Starting_Address : Word;
               Quantity_Of_Coils : Read_Coils_Quantity;
            when Read_Discrete_Inputs =>
               Read_Discrete_Inputs_Starting_Address : Word;
               Quantity_Of_Inputs : Read_Discrete_Inputs_Quantity;
            when Read_Holding_Registers =>
               Read_Holding_Registers_Starting_Address : Word;
               Quantity_Of_Registers_To_Read : Read_Holding_Registers_Quantity;
               Read_Holding_Registers_Data : PDU_Data;
            when Write_Multiple_Registers =>
               Write_Multiple_Registers_Starting_Address : Word;
               Quantity_Of_Registers_To_Write : Write_Multiple_Registers_Quantity;
               Write_Multiple_Registers_Data : PDU_Data;
         end case;
      end record;

   procedure Send (
                   Stream : not null access Ada.Streams.Root_Stream_Type'Class;
                   Item : In Protocol_Data_Unit);

   for Protocol_Data_Unit'Write use Send;

   procedure Receive (
                   Stream : not null access Ada.Streams.Root_Stream_Type'Class;
                   Item : out Protocol_Data_Unit);

   for Protocol_Data_Unit'Read use Receive;

   procedure Do_It;
end Modbus00;
