
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------

-- <summary>
-- This package provides functions to convert from one type to another.
-- </summary>
-- <description>
-- It provides :
-- - Bytes_To_Word,
-- - Word_To_Bytes,
-- - and many other.
-- </description>
-- <group>Libraries</group>

package A4A.Library.Conversion is
   procedure Bytes_To_Word (LSB_Byte : in Byte;
                            MSB_Byte : in Byte;
                            Word_out : out Word);

   procedure Word_To_Bytes (Word_in  : in Word;
                            LSB_Byte : out Byte;
                            MSB_Byte : out Byte);

   procedure Words_To_DWord (LSW_Word  : in Word;
                             MSW_Word  : in Word;
                             DWord_out : out DWord);

   procedure DWord_To_Words (DWord_in  : in DWord;
                             LSW_Word  : out Word;
                             MSW_Word  : out Word);

   procedure Byte_To_Booleans (Byte_in : in Byte;
                               Boolean_out00 : out Boolean;
                               Boolean_out01 : out Boolean;
                               Boolean_out02 : out Boolean;
                               Boolean_out03 : out Boolean;
                               Boolean_out04 : out Boolean;
                               Boolean_out05 : out Boolean;
                               Boolean_out06 : out Boolean;
                               Boolean_out07 : out Boolean
                              );

   procedure Booleans_To_Byte (
                               Boolean_in00 : in Boolean := false;
                               Boolean_in01 : in Boolean := false;
                               Boolean_in02 : in Boolean := false;
                               Boolean_in03 : in Boolean := false;
                               Boolean_in04 : in Boolean := false;
                               Boolean_in05 : in Boolean := false;
                               Boolean_in06 : in Boolean := false;
                               Boolean_in07 : in Boolean := false;
                               Byte_out : out Byte
                              );

   procedure Word_To_Booleans (Word_in : in Word;
                               Boolean_out00 : out Boolean;
                               Boolean_out01 : out Boolean;
                               Boolean_out02 : out Boolean;
                               Boolean_out03 : out Boolean;
                               Boolean_out04 : out Boolean;
                               Boolean_out05 : out Boolean;
                               Boolean_out06 : out Boolean;
                               Boolean_out07 : out Boolean;

                               Boolean_out08 : out Boolean;
                               Boolean_out09 : out Boolean;
                               Boolean_out10 : out Boolean;
                               Boolean_out11 : out Boolean;
                               Boolean_out12 : out Boolean;
                               Boolean_out13 : out Boolean;
                               Boolean_out14 : out Boolean;
                               Boolean_out15 : out Boolean
                              );

   procedure Booleans_To_Word (
                               Boolean_in00 : in Boolean := false;
                               Boolean_in01 : in Boolean := false;
                               Boolean_in02 : in Boolean := false;
                               Boolean_in03 : in Boolean := false;
                               Boolean_in04 : in Boolean := false;
                               Boolean_in05 : in Boolean := false;
                               Boolean_in06 : in Boolean := false;
                               Boolean_in07 : in Boolean := false;

                               Boolean_in08 : in Boolean := false;
                               Boolean_in09 : in Boolean := false;
                               Boolean_in10 : in Boolean := false;
                               Boolean_in11 : in Boolean := false;
                               Boolean_in12 : in Boolean := false;
                               Boolean_in13 : in Boolean := false;
                               Boolean_in14 : in Boolean := false;
                               Boolean_in15 : in Boolean := false;
                               Word_out : out Word
                              );

   procedure DWord_To_Booleans (DWord_in : in DWord;
                                Boolean_out00 : out Boolean;
                                Boolean_out01 : out Boolean;
                                Boolean_out02 : out Boolean;
                                Boolean_out03 : out Boolean;
                                Boolean_out04 : out Boolean;
                                Boolean_out05 : out Boolean;
                                Boolean_out06 : out Boolean;
                                Boolean_out07 : out Boolean;

                                Boolean_out08 : out Boolean;
                                Boolean_out09 : out Boolean;
                                Boolean_out10 : out Boolean;
                                Boolean_out11 : out Boolean;
                                Boolean_out12 : out Boolean;
                                Boolean_out13 : out Boolean;
                                Boolean_out14 : out Boolean;
                                Boolean_out15 : out Boolean;

                                Boolean_out16 : out Boolean;
                                Boolean_out17 : out Boolean;
                                Boolean_out18 : out Boolean;
                                Boolean_out19 : out Boolean;
                                Boolean_out20 : out Boolean;
                                Boolean_out21 : out Boolean;
                                Boolean_out22 : out Boolean;
                                Boolean_out23 : out Boolean;

                                Boolean_out24 : out Boolean;
                                Boolean_out25 : out Boolean;
                                Boolean_out26 : out Boolean;
                                Boolean_out27 : out Boolean;
                                Boolean_out28 : out Boolean;
                                Boolean_out29 : out Boolean;
                                Boolean_out30 : out Boolean;
                                Boolean_out31 : out Boolean
                               );

   procedure Booleans_To_DWord (
                                Boolean_in00 : in Boolean := false;
                                Boolean_in01 : in Boolean := false;
                                Boolean_in02 : in Boolean := false;
                                Boolean_in03 : in Boolean := false;
                                Boolean_in04 : in Boolean := false;
                                Boolean_in05 : in Boolean := false;
                                Boolean_in06 : in Boolean := false;
                                Boolean_in07 : in Boolean := false;

                                Boolean_in08 : in Boolean := false;
                                Boolean_in09 : in Boolean := false;
                                Boolean_in10 : in Boolean := false;
                                Boolean_in11 : in Boolean := false;
                                Boolean_in12 : in Boolean := false;
                                Boolean_in13 : in Boolean := false;
                                Boolean_in14 : in Boolean := false;
                                Boolean_in15 : in Boolean := false;

                                Boolean_in16 : in Boolean := false;
                                Boolean_in17 : in Boolean := false;
                                Boolean_in18 : in Boolean := false;
                                Boolean_in19 : in Boolean := false;
                                Boolean_in20 : in Boolean := false;
                                Boolean_in21 : in Boolean := false;
                                Boolean_in22 : in Boolean := false;
                                Boolean_in23 : in Boolean := false;

                                Boolean_in24 : in Boolean := false;
                                Boolean_in25 : in Boolean := false;
                                Boolean_in26 : in Boolean := false;
                                Boolean_in27 : in Boolean := false;
                                Boolean_in28 : in Boolean := false;
                                Boolean_in29 : in Boolean := false;
                                Boolean_in30 : in Boolean := false;
                                Boolean_in31 : in Boolean := false;
                                DWord_out : out DWord
                               );

end A4A.Library.Conversion;
