
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------


with Ada.Text_IO; use Ada.Text_IO;
with Ada.Real_Time; use Ada.Real_Time;

package body A4A.Library.Devices.Contactor is
   function Create (
                    Id : in String
                   )
                    return Instance is
      Device : Instance;
   begin
      Device.Status := Status_Off;
      Device.Id := new String'(Id);
      return Device;
   end Create;

   procedure Cyclic (
                     Device    : in out Instance;
                     Feed_Back : in Boolean;
                     Ack       : in Boolean;
                     Cmd_On    : in Boolean;
                     Coil      : out Boolean
                    ) is
      Elapsed : Time_Span;
      Tempo_Q : Boolean;
   begin

      Device.Tempo_Dis.Cyclic (
                               Start      => (Device.Status = Status_Going_On) or (Device.Status = Status_Going_Off),
                               Preset     => Milliseconds (1000),
                               Elapsed    => Elapsed,
                               Q          => Tempo_Q
                              );

      case Device.Status is
         when Status_Off =>
            if Cmd_On then
               Device.Status := Status_Going_On;
            elsif Feed_Back then --Immediate
               Device.Status := Status_Fault;
            end if;
         when Status_Going_On =>
            if Feed_Back then
               Device.Status := Status_On;
            elsif Tempo_Q then --Timer elapsed
               Device.Status := Status_Fault;
            end if;
         when Status_On =>
            if not Cmd_On then
               Device.Status := Status_Going_Off;
            elsif not Feed_Back then --Immediate
               Device.Status := Status_Fault;
            end if;
         when Status_Going_Off =>
            if not Feed_Back then
               Device.Status := Status_Off;
            elsif Tempo_Q then --Timer elapsed
               Device.Status := Status_Fault;
            end if;
         when Status_Fault =>
            if Ack then
               Device.Status := Status_Off;
            end if;
      end case;

      Coil := (Device.Status = Status_Going_On) or (Device.Status = Status_On);
   end Cyclic;

   function is_On (
                   Device    : in Instance
                  ) return Boolean is
   begin
      return Device.Status = Status_On;
   end is_On;

end A4A.Library.Devices.Contactor;
