
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------


with A4A.Library.Conversion;
with Ada.Text_IO;
with Ada.Integer_Text_IO;
with Ada.Long_Integer_Text_IO;

procedure Test_Convert is
   package Convert renames A4A.Library.Conversion;
   MyDWord : A4A.DWord;
   MyLSW : A4A.Word;
   MyMSW : A4A.Word;
   MyLSB : A4A.Byte;
   MyMSB : A4A.Byte;
   MyBooleans : array (0 .. 31) of Boolean;
begin
   MyDWord := 16#12345678#;
   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.DWord_To_Words (DWord_in => MyDWord, LSW_Word => MyLSW, MSW_Word => MyMSW);");
   Convert.DWord_To_Words (DWord_in => MyDWord, LSW_Word => MyLSW, MSW_Word => MyMSW);
   Ada.Text_IO.Put (Item => "MyDWord : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyDWord), Base => 16); Ada.Text_IO.Put (' ');
   Ada.Text_IO.Put (Item => "MyLSW : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyLSW), Base => 16); Ada.Text_IO.Put (' ');
   Ada.Text_IO.Put (Item => "MyMSW : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyMSW), Base => 16);
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.Words_To_DWord (LSW_Word => MyMSW, MSW_Word => MyLSW, DWord_out => MyDWord);");
   Convert.Words_To_DWord (LSW_Word => MyMSW, MSW_Word => MyLSW, DWord_out => MyDWord);
   Ada.Text_IO.Put (Item => "MyDWord : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyDWord), Base => 16);
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.Word_To_Bytes (Word_in => MyLSW, LSB_Byte => MyLSB, MSB_Byte => MyMSB);");
   Convert.Word_To_Bytes (Word_in => MyLSW, LSB_Byte => MyLSB, MSB_Byte => MyMSB);
   Ada.Text_IO.Put (Item => "MyLSB : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyLSB), Base => 16); Ada.Text_IO.Put (' ');
   Ada.Text_IO.Put (Item => "MyMSB : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyMSB), Base => 16);
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.Bytes_To_Word (LSB_Byte => MyMSB, MSB_Byte => MyLSB, Word_out => MyLSW);");
   Convert.Bytes_To_Word (LSB_Byte => MyMSB, MSB_Byte => MyLSB, Word_out => MyLSW);
   Ada.Text_IO.Put (Item => "MyLSW : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyLSW), Base => 16);
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.Byte_To_Booleans (Byte_in => MyLSB, ...);");
   Convert.Byte_To_Booleans (Byte_in => MyLSB,
                            Boolean_out00 => MyBooleans(0),
                            Boolean_out01 => MyBooleans(1),
                            Boolean_out02 => MyBooleans(2),
                            Boolean_out03 => MyBooleans(3),
                            Boolean_out04 => MyBooleans(4),
                            Boolean_out05 => MyBooleans(5),
                            Boolean_out06 => MyBooleans(6),
                            Boolean_out07 => MyBooleans(7)
                              );
   Ada.Text_IO.Put (Item => "MyLSB : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyLSB), Base => 16);
   Ada.Text_IO.New_Line;
   Ada.Text_IO.Put (Item => "MyBooleans(0 .. 7) reverse : ");
   for Index in reverse 0 .. 7 loop
      Ada.Text_IO.Put (Item => MyBooleans(Index)'Img & ' ');
   end loop;
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.Booleans_To_Byte (Byte_out => MyLSB, ...);");
   MyBooleans(0) := false;
   MyBooleans(1) := true;
   MyBooleans(2) := true;
   MyBooleans(3) := true;  --E
   MyBooleans(4) := true;
   MyBooleans(5) := true;
   MyBooleans(6) := true;
   MyBooleans(7) := false; --7
   Ada.Text_IO.Put (Item => "MyBooleans(0 .. 7) reverse : ");
   for Index in reverse 0 .. 7 loop
      Ada.Text_IO.Put (Item => MyBooleans(Index)'Img & ' ');
   end loop;
   Ada.Text_IO.New_Line;
   Convert.Booleans_To_Byte (Boolean_in00 => MyBooleans(0),
                             Boolean_in01 => MyBooleans(1),
                             Boolean_in02 => MyBooleans(2),
                             Boolean_in03 => MyBooleans(3),
                             Boolean_in04 => MyBooleans(4),
                             Boolean_in05 => MyBooleans(5),
                             Boolean_in06 => MyBooleans(6),
                             Boolean_in07 => MyBooleans(7),
                             Byte_out => MyLSB);
   Ada.Text_IO.Put (Item => "MyLSB : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyLSB), Base => 16); Ada.Text_IO.Put (' ');
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.Word_To_Booleans (Word_in => MyLSW, ...);");
   Convert.Word_To_Booleans (Word_in => MyLSW,
                            Boolean_out00 => MyBooleans(0),
                            Boolean_out01 => MyBooleans(1),
                            Boolean_out02 => MyBooleans(2),
                            Boolean_out03 => MyBooleans(3),
                            Boolean_out04 => MyBooleans(4),
                            Boolean_out05 => MyBooleans(5),
                            Boolean_out06 => MyBooleans(6),
                            Boolean_out07 => MyBooleans(7),

                            Boolean_out08 => MyBooleans(8),
                            Boolean_out09 => MyBooleans(9),
                            Boolean_out10 => MyBooleans(10),
                            Boolean_out11 => MyBooleans(11),
                            Boolean_out12 => MyBooleans(12),
                            Boolean_out13 => MyBooleans(13),
                            Boolean_out14 => MyBooleans(14),
                            Boolean_out15 => MyBooleans(15)
                              );
   Ada.Text_IO.Put (Item => "MyLSW : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyLSW), Base => 16); Ada.Text_IO.Put (' ');
   Ada.Text_IO.Put_Line (Item => "MyBooleans(0 .. 15) reverse : ");
   for Index in reverse 0 .. 15 loop
      Ada.Text_IO.Put (Item => MyBooleans(Index)'Img & ' ');
      if Index = 8 then
         Ada.Text_IO.New_Line;
      end if;
   end loop;
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.Booleans_To_Word (Word_out => MyLSW, ...);");
   Convert.Booleans_To_Word (
                             Boolean_in00 => true,
                             Boolean_in01 => true,
                             Boolean_in02 => true,
                             Boolean_in03 => true, --F
                             Boolean_in04 => false,
                             Boolean_in05 => true,
                             Boolean_in06 => true,
                             Boolean_in07 => true, --E

                             Boolean_in08 => false,
                             Boolean_in09 => true,
                             Boolean_in10 => true,
                             Boolean_in11 => true, --E
                             Boolean_in12 => true,
                             Boolean_in13 => true,
                             Boolean_in14 => false,
                             Boolean_in15 => true, --B
                             Word_out => MyLSW);
   Ada.Text_IO.Put (Item => "MyLSW : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyLSW), Base => 16);
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.DWord_To_Booleans (DWord_in => MyDWord, ...);");
   Convert.DWord_To_Booleans (DWord_in => MyDWord,
                            Boolean_out00 => MyBooleans(0),
                            Boolean_out01 => MyBooleans(1),
                            Boolean_out02 => MyBooleans(2),
                            Boolean_out03 => MyBooleans(3),
                            Boolean_out04 => MyBooleans(4),
                            Boolean_out05 => MyBooleans(5),
                            Boolean_out06 => MyBooleans(6),
                            Boolean_out07 => MyBooleans(7),

                            Boolean_out08 => MyBooleans(8),
                            Boolean_out09 => MyBooleans(9),
                            Boolean_out10 => MyBooleans(10),
                            Boolean_out11 => MyBooleans(11),
                            Boolean_out12 => MyBooleans(12),
                            Boolean_out13 => MyBooleans(13),
                            Boolean_out14 => MyBooleans(14),
                            Boolean_out15 => MyBooleans(15),

                            Boolean_out16 => MyBooleans(16),
                            Boolean_out17 => MyBooleans(17),
                            Boolean_out18 => MyBooleans(18),
                            Boolean_out19 => MyBooleans(19),
                            Boolean_out20 => MyBooleans(20),
                            Boolean_out21 => MyBooleans(21),
                            Boolean_out22 => MyBooleans(22),
                            Boolean_out23 => MyBooleans(23),

                            Boolean_out24 => MyBooleans(24),
                            Boolean_out25 => MyBooleans(25),
                            Boolean_out26 => MyBooleans(26),
                            Boolean_out27 => MyBooleans(27),
                            Boolean_out28 => MyBooleans(28),
                            Boolean_out29 => MyBooleans(29),
                            Boolean_out30 => MyBooleans(30),
                            Boolean_out31 => MyBooleans(31)
                             );
   Ada.Text_IO.Put (Item => "MyDWord : ");
   Ada.Integer_Text_IO.Put (Item => Integer(MyDWord), Base => 16); Ada.Text_IO.Put (' ');
   Ada.Text_IO.Put_Line (Item => "MyBooleans(0 .. 31) reverse : ");
   for Index in reverse 0 .. 31 loop
      Ada.Text_IO.Put (Item => MyBooleans(Index)'Img & ' ');
      if Index = 8 or Index = 16 or Index = 24 then
         Ada.Text_IO.New_Line;
      end if;
   end loop;
   Ada.Text_IO.New_Line(2);

   Ada.Text_IO.Put_Line (Item => " -- " & "Convert.Booleans_To_DWord (DWord_out => MyDWord, ...);");
   Convert.Booleans_To_DWord (
                             Boolean_in00 => true,
                             Boolean_in01 => true,
                             Boolean_in02 => true,
                             Boolean_in03 => true, --F
                             Boolean_in04 => false,
                             Boolean_in05 => true,
                             Boolean_in06 => true,
                             Boolean_in07 => true, --E

                             Boolean_in08 => false,
                             Boolean_in09 => true,
                             Boolean_in10 => true,
                             Boolean_in11 => true, --E
                             Boolean_in12 => true,
                             Boolean_in13 => true,
                             Boolean_in14 => false,
                             Boolean_in15 => true, --B

                             Boolean_in16 => true,
                             Boolean_in17 => false,
                             Boolean_in18 => true,
                             Boolean_in19 => true, --D
                             Boolean_in20 => false,
                             Boolean_in21 => true,
                             Boolean_in22 => false,
                             Boolean_in23 => true, --A

                             Boolean_in24 => false,
                             Boolean_in25 => true,
                             Boolean_in26 => true,
                             Boolean_in27 => true, --E
                             Boolean_in28 => true,
                             Boolean_in29 => false,
                             Boolean_in30 => true,
                             Boolean_in31 => true, --D
                             DWord_out => MyDWord);
   Ada.Text_IO.Put (Item => "MyDWord : ");
   --Ada.Integer_Text_IO.Put (Item => Integer(MyDWord), Base => 16);
   Ada.Long_Integer_Text_IO.Put (Item => Long_Integer(MyDWord), Base => 16);
   Ada.Text_IO.New_Line(2);

end Test_Convert;


