
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------


with System;
with Interfaces.C.Strings;

with Ada.Text_IO;
with Ada.Integer_Text_IO;
with Ada.Exceptions; use Ada.Exceptions;

with A4A; use A4A;
with A4A.Protocols.LibModbus;

procedure Test_LibModbus_Client is
   package LibModbus renames A4A.Protocols.LibModbus;
   MyContext : LibModbus.Context_Type;
   Registers : Word_Array(0 .. 9);

   Context_Ok : Boolean := False;
   Connect_Ok : Boolean := False;
   procedure Close is
   begin
      if Connect_Ok then
         LibModbus.Close (Context => MyContext);
      end if;
      if Context_Ok then
         LibModbus.Free (Context => MyContext);
      end if;
      Ada.Text_IO.Put(Item => "Closing gracefully.");
   end Close;

begin
   Ada.Text_IO.Put_Line(Item => "Test_LibModbus_Client...");

   MyContext := LibModbus.New_TCP(IP_Address => "127.0.0.1",
                                  Port       => 1502);
   Context_Ok := True;

   LibModbus.Connect(Context => MyContext);
   Connect_Ok := True;

   for I in 1 .. 10 loop
      LibModbus.Read_Registers(Context => MyContext,
                               Offset  => 0,
                               Number  => Registers'Length,
                               Dest    => Registers'Address);
   end loop;

   --   for I in Registers'First .. Registers'Last loop
   for I in Registers'Range loop
      Ada.Text_IO.Put(Item => "Registers(" & I'Img & ") := ");
      Ada.Integer_Text_IO.Put(Item => Integer(Registers(I)), Width => 8, Base => 16);
      Ada.Text_IO.New_Line;
   end loop;

   Close;

exception
   when Error: LibModbus.Context_Error | LibModbus.Connect_Error =>
      Ada.Text_IO.Put_Line ("libmodbus exception: ");
      Ada.Text_IO.Put_Line (Exception_Information(Error));
      Close;

   when Error: others =>
      Ada.Text_IO.Put_Line ("Unexpected exception: ");
      Ada.Text_IO.Put_Line (Exception_Information(Error));
      Close;

end Test_LibModbus_Client;


