
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------

--  #define NB_CONNECTION    5
--
--  modbus_t *ctx = NULL;
--  int server_socket;
--  modbus_mapping_t *mb_mapping;
--
--  static void close_sigint(int dummy)
--  {
--      close(server_socket);
--      modbus_free(ctx);
--      modbus_mapping_free(mb_mapping);
--
--      exit(dummy);
--  }
--
--  int main(void)
--  {
--      int master_socket;
--      int rc;
--      fd_set refset;
--      fd_set rdset;
--
--      /* Maximum file descriptor number */
--      int fdmax;
--
--      ctx = modbus_new_tcp("127.0.0.1", 1502);
--
--      mb_mapping = modbus_mapping_new(MODBUS_MAX_READ_BITS, 0,
--                                      MODBUS_MAX_READ_REGISTERS, 0);
--      if (mb_mapping == NULL) {
--          fprintf(stderr, "Failed to allocate the mapping: %s\n",
--                  modbus_strerror(errno));
--          modbus_free(ctx);
--          return -1;
--      }
--
--      server_socket = modbus_tcp_listen(ctx, NB_CONNECTION);
--
--      signal(SIGINT, close_sigint);
--
--      /* Clear the reference set of socket */
--      FD_ZERO(&refset);
--      /* Add the server socket */
--      FD_SET(server_socket, &refset);
--
--      /* Keep track of the max file descriptor */
--      fdmax = server_socket;
--
--      for (;;) {
--          rdset = refset;
--          if (select(fdmax+1, &rdset, NULL, NULL, NULL) == -1) {
--              perror("Server select() failure.");
--              close_sigint(1);
--          }
--
--          /* Run through the existing connections looking for data to be
--           * read */
--          for (master_socket = 0; master_socket <= fdmax; master_socket++) {
--
--              if (FD_ISSET(master_socket, &rdset)) {
--                  if (master_socket == server_socket) {
--                      /* A client is asking a new connection */
--                      socklen_t addrlen;
--                      struct sockaddr_in clientaddr;
--                      int newfd;
--
--                      /* Handle new connections */
--                      addrlen = sizeof(clientaddr);
--                      memset(&clientaddr, 0, sizeof(clientaddr));
--                      newfd = accept(server_socket, (struct sockaddr *)&clientaddr, &addrlen);
--                      if (newfd == -1) {
--                          perror("Server accept() error");
--                      } else {
--                          FD_SET(newfd, &refset);
--
--                          if (newfd > fdmax) {
--                              /* Keep track of the maximum */
--                              fdmax = newfd;
--                          }
--                          printf("New connection from %s:%d on socket %d\n",
--                                 inet_ntoa(clientaddr.sin_addr), clientaddr.sin_port, newfd);
--                      }
--                  } else {
--                      /* An already connected master has sent a new query */
--                      uint8_t query[MODBUS_TCP_MAX_ADU_LENGTH];
--
--                      modbus_set_socket(ctx, master_socket);
--                      rc = modbus_receive(ctx, query);
--                      if (rc != -1) {
--                          modbus_reply(ctx, query, rc, mb_mapping);
--                      } else {
--                          /* Connection closed by the client, end of server */
--                          printf("Connection closed on socket %d\n", master_socket);
--                          close(master_socket);
--
--                          /* Remove from reference set */
--                          FD_CLR(master_socket, &refset);
--
--                          if (master_socket == fdmax) {
--                              fdmax--;
--                          }
--                      }
--                  }
--              }
--          }
--      }
--
--      return 0;
--  }
with System;
with Interfaces.C.Strings;
use Interfaces.C;

with Ada.Text_IO;
with Ada.Exceptions; use Ada.Exceptions;

with GNAT.Sockets; use GNAT.Sockets;
--  with GNAT.Sockets.Thin_Common;
--  with GNAT.Sockets.Thin;
with GNAT.Ctrl_C; use GNAT.Ctrl_C;

with A4A; use A4A;
with A4A.Protocols.LibModbus;

package body Test_LibModbus_Server_Many_Package is

   Quit       : Boolean := False;

   procedure Ctrl_C is
   begin
      Quit := True;
   end Ctrl_C;

   procedure Test_LibModbus_Server_Many is
      package LibModbus renames A4A.Protocols.LibModbus;
      MyContext         : LibModbus.Context_Type;
      Result            : Interfaces.C.int;
      Req_Len           : Interfaces.C.int;
      Request           : aliased Byte_Array (Integer range 0 .. LibModbus.MODBUS_TCP_MAX_ADU_LENGTH);

      Server_Socket     : LibModbus.Socket_Type;
      Master_Socket     : LibModbus.Socket_Type;

      refset            : Socket_Set_Type;
      rdset             : Socket_Set_Type;
      wtset             : Socket_Set_Type;

      Selector          : Selector_Type;
      Status            : Selector_Status;

      MyMapping_Access  : System.Address;

      Loop1_Count       : Integer := 0;
      Loop2_Count       : Integer := 0;

      Time_Out          : constant:= 10.0;     -- 1s

      Context_Ok : Boolean := False;
      Connect_Ok : Boolean := False;
      Mapping_Ok : Boolean := False;
      procedure Close is
      begin
         if Mapping_Ok then
            LibModbus.Mapping_Free (Mapping_Access => MyMapping_Access);
         end if;
         if Connect_Ok then
            LibModbus.Close (Context => MyContext);
         end if;
         if Context_Ok then
            LibModbus.Free (Context => MyContext);
         end if;

         Ada.Text_IO.Put(Item => "Closing gracefully.");
         Uninstall_Handler;
      end Close;

   begin
      Ada.Text_IO.Put(Item => "Test_LibModbus_Server_Many... Waiting");
      Ada.Text_IO.New_Line;

      Install_Handler (Ctrl_C'Access);

      MyContext := LibModbus.New_TCP(IP_Address => "127.0.0.1",
                                     Port       => 1502);
      Context_Ok := True;

      LibModbus.Set_Debug(Context => MyContext, On => True);

      MyMapping_Access := LibModbus.Mapping_New (Nb_Coil_Status       => 80,
                                                 Nb_Input_Status      => 80,
                                                 Nb_Holding_Registers => 10,
                                                 Nb_Input_Registers   => 10);
      Mapping_Ok := True;

      declare
         MyMapping : LibModbus.Modbus_Mapping_Type;
         for MyMapping'Address use MyMapping_Access;
         My_tab_registers : Word_Array(0..9);
         for My_tab_registers'Address use MyMapping.tab_registers;
      begin
         My_tab_registers(0) := 16#0102#;
         My_tab_registers(1) := 16#0304#;
         My_tab_registers(2) := 16#0506#;
         My_tab_registers(3) := 16#0708#;
         My_tab_registers(4) := 16#090A#;
         My_tab_registers(5) := 16#0B0C#;
         My_tab_registers(6) := 16#FF0C#;
         My_tab_registers(7) := 16#FF0B#;
         My_tab_registers(8) := 16#FF0A#;
         My_tab_registers(9) := 16#FFFF#;

         Server_Socket := LibModbus.TCP_Listen(Context       => MyContext,
                                               Nb_Connection => 2);
         -- Wait forever a Client connection
         Connect_Ok := True;

         Ada.Text_IO.Put_Line(Item => "TCP_Listen... Server_Socket = " & Image(Server_Socket));
         Empty(refset);

         Set(refset, Server_Socket);

         Create_Selector(Selector);

         loop

            Ada.Text_IO.Put_Line(Item => "Loop1_Count = " & Loop1_Count'Img);

            Copy(refset, rdset);

            Check_Selector(Selector, rdset, wtset, Status, Time_Out);

            case Status is
            when Completed =>

               Ada.Text_IO.Put_Line(Item => "Check_Selector... Status = Completed");

               if Is_Set(rdset, Server_Socket) then
                  -- A client is asking a new connection
                  Ada.Text_IO.Put_Line(Item => "A client is asking a new connection");

                  LibModbus.TCP_Accept(Context       => MyContext,
                                       Socket_Access => Server_Socket'Address);

                  Master_Socket := LibModbus.Get_Socket(Context => MyContext);

                  Ada.Text_IO.Put_Line(Item => "TCP_Accept... Master_Socket = " & Image(Master_Socket));

                  Set(refset, Master_Socket);
               else
                  loop
                     Ada.Text_IO.Put_Line(Item => "Loop2_Count = " & Loop2_Count'Img);

                     Get(rdset, Master_Socket);
                     exit when Master_Socket = No_Socket;

                     -- An already connected master has sent a new query
                     Ada.Text_IO.Put_Line(Item => "An already connected master has sent a new query");

                     LibModbus.Set_Socket(Context => MyContext, Socket => Master_Socket);

                     Req_Len := LibModbus.Receive(Context => MyContext,
                                                  Request => Request'Address);
                     if (Req_Len > 0) then
                        Ada.Text_IO.Put_Line(Item => "Request received");

                        Result := LibModbus.Reply(Context        => MyContext,
                                                  Request        => Request'Address,
                                                  Request_Len    => Req_Len,
                                                  Mapping_Access => MyMapping_Access);
                     else
                        -- Connection closed by the client
                        Ada.Text_IO.Put_Line(Item => "Connection closed by the client");

                        Close_Socket(Master_Socket);
                        Clear(refset, Master_Socket);
                     end if;

                     Loop2_Count := Loop2_Count + 1;

                  end loop;
               end if;

            when Expired =>
               Ada.Text_IO.Put_Line(Item => "Check_Selector... Status = Expired");
            when Aborted =>
               Ada.Text_IO.Put_Line(Item => "Check_Selector... Status = Aborted");
            end case;

            Loop1_Count := Loop1_Count + 1;

            exit when Quit;

         end loop;
      end;

      Close;

   exception
      when Error: LibModbus.Context_Error | LibModbus.Connect_Error =>
         Ada.Text_IO.Put_Line ("libmodbus exception: ");
         Ada.Text_IO.Put_Line (Exception_Information(Error));
         Close;

      when Error: others =>
         Ada.Text_IO.Put_Line ("Unexpected exception: ");
         Ada.Text_IO.Put_Line (Exception_Information(Error));
         Close;

   end Test_LibModbus_Server_Many;

end Test_LibModbus_Server_Many_Package;
