
-----------------------------------------------------------------------
--                       Ada for Automation                          --
--                                                                   --
--                 Copyright (C) 2012, Stephane LOS                  --
--                                                                   --
-- This library is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This library is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this library; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-- As a special exception, if other files instantiate generics from  --
-- this unit, or you link this unit with other files to produce an   --
-- executable, this  unit  does not  by itself cause  the resulting  --
-- executable to be covered by the GNU General Public License. This  --
-- exception does not however invalidate any other reasons why the   --
-- executable file  might be covered by the  GNU Public License.     --
-----------------------------------------------------------------------


with System;
with Interfaces.C.Strings;
use Interfaces.C;

with Ada.Text_IO;
with Ada.Exceptions; use Ada.Exceptions;

with A4A; use A4A;
with A4A.Protocols.LibModbus;

procedure Test_LibModbus_Server_Single is
   package LibModbus renames A4A.Protocols.LibModbus;
   MyContext         : LibModbus.Context_Type;
   Result            : Interfaces.C.int;
   Req_Len           : Interfaces.C.int;
   Request           : aliased Byte_Array (Integer range 0 .. LibModbus.MODBUS_TCP_MAX_ADU_LENGTH);
   Server_Socket     : aliased LibModbus.Socket_Type;
   MyMapping_Access  : System.Address;

   Context_Ok : Boolean := False;
   Connect_Ok : Boolean := False;
   Mapping_Ok : Boolean := False;
   procedure Close is
   begin
      if Mapping_Ok then
         LibModbus.Mapping_Free (Mapping_Access => MyMapping_Access);
      end if;
      if Connect_Ok then
         LibModbus.Close (Context => MyContext);
      end if;
      if Context_Ok then
         LibModbus.Free (Context => MyContext);
      end if;
      Ada.Text_IO.Put(Item => "Closing gracefully.");
   end Close;

begin
   Ada.Text_IO.Put(Item => "Test_LibModbus_Server_Single... Waiting");
   Ada.Text_IO.New_Line;
   MyContext := LibModbus.New_TCP(IP_Address => "127.0.0.1",
                                  Port       => 1502);
   Context_Ok := True;

   LibModbus.Set_Debug(Context => MyContext, On => True);

   MyMapping_Access := LibModbus.Mapping_New (Nb_Coil_Status       => 80,
                                              Nb_Input_Status      => 80,
                                              Nb_Holding_Registers => 10,
                                              Nb_Input_Registers   => 10);
   Mapping_Ok := True;

   declare
      MyMapping : LibModbus.Modbus_Mapping_Type;
      for MyMapping'Address use MyMapping_Access;
      My_tab_registers : Word_Array(0..9);
      for My_tab_registers'Address use MyMapping.tab_registers;
   begin
      My_tab_registers(0) := 16#0102#;
      My_tab_registers(1) := 16#0304#;
      My_tab_registers(2) := 16#0506#;
      My_tab_registers(3) := 16#0708#;
      My_tab_registers(4) := 16#090A#;
      My_tab_registers(5) := 16#0B0C#;
      My_tab_registers(6) := 16#FF0C#;
      My_tab_registers(7) := 16#FF0B#;
      My_tab_registers(8) := 16#FF0A#;
      My_tab_registers(9) := 16#FFFF#;

      Server_Socket := LibModbus.TCP_Listen(Context       => MyContext,
                                            Nb_Connection => 1);
      LibModbus.TCP_Accept(Context       => MyContext,
                           Socket_Access => Server_Socket'Address);
      -- Wait forever a Client connection
      Connect_Ok := True;

      loop
         Req_Len := LibModbus.Receive(Context => MyContext,
                                      Request => Request'Address);
         if (Req_Len > 0) then
            Result := LibModbus.Reply(Context        => MyContext,
                                      Request        => Request'Address,
                                      Request_Len    => Req_Len,
                                      Mapping_Access => MyMapping_Access);
         else
            -- The Client has gone
            exit;
         end if;
      end loop;
   end;

   Close;

exception
   when Error: LibModbus.Context_Error | LibModbus.Connect_Error =>
      Ada.Text_IO.Put_Line ("libmodbus exception: ");
      Ada.Text_IO.Put_Line (Exception_Information(Error));
      Close;

   when Error: others =>
      Ada.Text_IO.Put_Line ("Unexpected exception: ");
      Ada.Text_IO.Put_Line (Exception_Information(Error));
      Close;

end Test_LibModbus_Server_Single;


